#ifndef ACTOR_H
#define ACTOR_H

#include <cpctelera.h>
#include "Map.h"
#include "Object.h"
#include "Utility.h"

#define ACTOR_TYPE_PLAYER 1
#define ACTOR_TYPE_ENEMY 2

typedef struct Actor Actor;

struct Actor
{
    // Object containing position and velocity.
    Object object;

    // The x position of the link activated by the actor.
    u8 activatedLinkX;

    // The y position of the link activated by the actor.
    u8 activatedLinkY;

    // Number of hitpoints.
    u8 health;

    // Unique identifier for identifying the different actors in the system.
    u8 id;

    // Indicates whether the actor should be redrawn on screen.
    u8 redraw;

    // The type of actor (player, enemy, block).
    u8 actorType;
};

// Draws the actor on screen making use of the backbuffer to reduce flicker.
void drawActor(Actor* actor, u8* actorSprite);

// Draw the actor object on screen without making use of the backbuffer.
void drawActorNoBackbuffer(Object* object, u8* actorSprite);

// Activates\deactivates an interactable connected to the link that the actor is currently positioned on top of.
void activateActorLink(Actor* actor, u8 currentLinkX, u8 currentLinkY);

// Indicates how many cells the actor of the specified cell type.
u8 actorOccuppyingCells(Actor* actor, u8 cellType);

// The next unique id to use for actor identification in the system.
u8 nextActorId();

// Initialises common actor variables in the actor struct.
void initActor(Actor* actor, u8 x, u8 y, u8 actorType);

#endif