#include "Enemy.h"

void initEnemies(u8* enemiesArray, u8 arraySize)
{
    u8 i = 0;
    Enemy* enemy = mapEnemies;

    for (i = 0; i < arraySize; i += ENEMY_ARRAY_COMPONENT_COUNT)
    {
        u8 x = enemiesArray[i];
        u8 y = enemiesArray[i + 1];

        // Initialise actor variables.
        initActor((Actor*)enemy, x, y, ACTOR_TYPE_ENEMY);

        enemy->type = enemiesArray[i + 2];
        enemy->health = 3;

        if (enemy->type == ENEMY_UP_DOWN || enemy->type == ENEMY_CORNERING)
        {
            // Initial velocity for up down and cornering enemies.
            enemy->object.velocity.y = 4;
        }
        else if (enemy->type == ENEMY_LEFT_RIGHT)
        {
            // Initial velocity for left right enemy.
            enemy->object.velocity.x = 2;
        }
        else if (enemy->type <= ENEMY_MOVING_ROCK_DOWN)
        {
            // Make sure the moving rock can never spawn a projectile but we can reuse the turret code for moving rocks.
            enemy->projectileIndex = 255;

            // The player shouldn't be able to kill these enemies so set to maximum health.
            enemy->health = 255;
        }
        else if (enemy->type == ENEMY_SHARK)
        {
            // Sharks require more shots to kill so you probably shouldn't try.
            enemy->health = 5;
        }
        else if (enemy->type > ENEMY_SHARK)
        {
            // The player shouldn't be able to kill these enemies so set to maximum health.
            enemy->health = 255;

            // TODO update if statement on adding additional enemy types.
            // Enemy spike and turret should never be drawn or redrawn.
            enemy->redraw = FALSE;

            // Save the direction of the turret for spawning projectiles.
            if (enemy->type == ENEMY_TURRET_RIGHT)
            {
                enemy->direction = DIRECTION_RIGHT;
            }
            else if (enemy->type == ENEMY_TURRET_LEFT)
            {
                enemy->direction = DIRECTION_LEFT;
            }
            else if (enemy->type == ENEMY_TURRET_UP)
            {
                enemy->direction = DIRECTION_UP;
            }
            else if (enemy->type == ENEMY_TURRET_DOWN)
            {
                enemy->direction = DIRECTION_DOWN;
            }
        }

        enemy->logicFlag = 0;

        // Move to the next element of the map enemy array.
        ++enemy;
    }
}

void drawEnemies()
{
    u8 i = 0;
    Enemy* enemy = mapEnemies;

    for (i = 0; i < mapEnemyCount; ++i)
    {
        if (enemy->object.position.room == currentRoom && enemy->redraw)
        {
            // Types start from 1 instead of 0 so we nee the -1.
            drawActor((Actor*)enemy, enemyGraphics[enemy->type - 1]);

            enemy->redraw = FALSE;
        }

        ++enemy;
    }
}

void markEnemiesForRedraw()
{
    u8 i = 0;

    Enemy* enemy = mapEnemies;

    for (i = 0; i < mapEnemyCount; ++i)
    {
        // Only mark for redraw enemies in the current room.
        if (enemy->object.position.room == currentRoom)
        {
            // Enemy spike and turrets should never be drawn or redrawn.
            if (enemy->type < ENEMY_TURRET_LEFT)
            {
                // TODO update if statement on adding additional enemy types.
                enemy->redraw = TRUE;
            }
        }

        ++enemy;
    }
}

void logicEnemies()
{
    u8 i = 0;

    Enemy* enemy = mapEnemies;

    // Seed srand with the current time.
    cpct_srand(levelTimerSecondsRemaining);

    for (i = 0; i < mapEnemyCount; ++i)
    {
        // Only run logic on enemy if it is on screen (in the current room).
        if (enemy->object.position.room == currentRoom)
        {
            // Check if the current enemy is moving.
            if (enemy->object.velocity.x || enemy->object.velocity.y)
            {
                // Using the logic flag to reduce the frequency of running the enemy logic to improve performance.
                // This results in the enemy being drawn less, unfortunately the enemy looks less smooth doing this as well.
                enemy->logicFlag++;

                if (enemy->logicFlag > 2)
                {
                    // Save the link index before resetting it.
                    u8 currentLinkX = enemy->activatedLinkX;
                    u8 currentLinkY = enemy->activatedLinkY;

                    // Reset link X to determine if the enemy has moved causing the enemy to move off the source of the link.
                    enemy->activatedLinkX = NO_LINK_FOUND;

                    updateEnemyPosition(enemy);

                    // Determine whether the link should be activated or deactivated based on whether the enemy is moving onto the plate or off it.
                    activateActorLink((Actor*)enemy, currentLinkX, currentLinkY);

                    enemy->logicFlag = 0;

                    // If enemy redraw is false then the enemy has not moved.
                    if (enemy->redraw == FALSE)
                    {
                        // Up and down enemy.
                        if (enemy->type == ENEMY_UP_DOWN)
                        {
                            // Enemy has been blocked so reverse the velocity.
                            enemy->object.velocity.y *= -1;
                        }
                        else if (enemy->type == ENEMY_LEFT_RIGHT)
                        {
                            // Enemy has been blocked so reverse the velocity.
                            enemy->object.velocity.x *= -1;
                        }
                        else if (enemy->type == ENEMY_MOVING_ROCK_LEFT)
                        {
                            if (enemy->object.velocity.x == -3)
                            {
                                // Slow down the enemy at the cell edge to make sure it gets all the way to the edge.
                                enemy->object.velocity.x = -1;
                            }
                            else
                            {
                                // Convert the enemy to a timed spike (enemies don't have health so this is my current solution to destroying the moving rock).
                                convertEnemyToSpike(enemy);
                            }
                        }
                        else if (enemy->type == ENEMY_MOVING_ROCK_RIGHT)
                        {
                            if (enemy->object.velocity.x == 3)
                            {
                                // Slow down the enemy at the cell edge to make sure it gets all the way to the edge.
                                enemy->object.velocity.x = 1;
                            }
                            else
                            {
                                // Convert the enemy to a timed spike (enemies don't have health so this is my current solution to destroying the moving rock).
                                convertEnemyToSpike(enemy);
                            }
                        }
                        else if (enemy->type == ENEMY_MOVING_ROCK_UP)
                        {
                            if (enemy->object.velocity.y == -6)
                            {
                                // Slow down the enemy at the cell edge to make sure it gets all the way to the edge.
                                enemy->object.velocity.y = -2;
                            }
                            else
                            {
                                // Convert the enemy to a timed spike (enemies don't have health so this is my current solution to destroying the moving rock).
                                convertEnemyToSpike(enemy);
                            }
                        }
                        else if (enemy->type == ENEMY_MOVING_ROCK_DOWN)
                        {
                            if (enemy->object.velocity.y == 6)
                            {
                                // Slow down the enemy at the cell edge to make sure it gets all the way to the edge.
                                enemy->object.velocity.y = 2;
                            }
                            else
                            {
                                // Convert the enemy to a timed spike (enemies don't have health so this is my current solution to destroying the moving rock).
                                convertEnemyToSpike(enemy);
                            }
                        }
                        else if (enemy->type == ENEMY_CORNERING)
                        {
                            // Cornering enemies change direction ninety degrees every time they collide with a solid object.
                            if (enemy->object.velocity.y == 4)
                            {
                                // Change direction from down to left.
                                enemy->object.velocity.y = 0;
                                enemy->object.velocity.x = -2;
                            }
                            else if (enemy->object.velocity.x == -2)
                            {
                                // Change direction from left to up.
                                enemy->object.velocity.y = -4;
                                enemy->object.velocity.x = 0;
                            }
                            else if (enemy->object.velocity.y == -4)
                            {
                                // Change direction from up to right.
                                enemy->object.velocity.y = 0;
                                enemy->object.velocity.x = 2;
                            }
                            else if (enemy->object.velocity.x == 2)
                            {
                                // Change direction from right to down.
                                enemy->object.velocity.y = 4;
                                enemy->object.velocity.x = 0;
                            }
                        }
                    }
                }
            }
            else
            {
                // Turrets never move.
                if (enemy->type == ENEMY_TURRET_RIGHT || enemy->type == ENEMY_MOVING_ROCK_RIGHT)
                {
                    // Check that the player is in range of the turret.
                    if (enemy->object.position.x <= player.object.position.x)
                    {
                        if (difference(enemy->object.position.y, player.object.position.y) < 64)
                        {
                            // Only spawn a projectile if the enemy doesn't currently have an active projectile.
                            if (enemy->projectileIndex == 0)
                            {
                                // Spawn projectile at the enemy turret position that is travelling right.
                                spawnEnemyProjectile(enemy, i);
                            }
                            else if (enemy->type == ENEMY_MOVING_ROCK_RIGHT)
                            {
                                enemy->object.velocity.x = 3;
                            }
                        }
                    }
                }
                else if (enemy->type == ENEMY_TURRET_LEFT || enemy->type == ENEMY_MOVING_ROCK_LEFT)
                {
                    // Check that the player is in range of the turret.
                    if (enemy->object.position.x >= player.object.position.x)
                    {
                        if (difference(enemy->object.position.y, player.object.position.y) < 64)
                        {
                            // Only spawn a projectile if the enemy doesn't currently have an active projectile.
                            if (enemy->projectileIndex == 0)
                            {
                                // Spawn projectile at the enemy turret position that is travelling left.
                                spawnEnemyProjectile(enemy, i);
                            }
                            else if (enemy->type == ENEMY_MOVING_ROCK_LEFT)
                            {
                                enemy->object.velocity.x = -3;
                            }
                        }
                    }
                }
                else if (enemy->type == ENEMY_TURRET_DOWN || enemy->type == ENEMY_MOVING_ROCK_DOWN)
                {
                    if (enemy->object.position.y <= player.object.position.y)
                    {
                        if (difference(enemy->object.position.x, player.object.position.x) < 12)
                        {
                            // Only spawn a projectile if the enemy doesn't currently have an active projectile.
                            if (enemy->projectileIndex == 0)
                            {
                                // Spawn projectile at the enemy turret position that is travelling downward.
                                spawnEnemyProjectile(enemy, i);
                            }
                            else if (enemy->type == ENEMY_MOVING_ROCK_DOWN)
                            {
                                enemy->object.velocity.y = 6;
                            }
                        }
                    }
                }
                else if (enemy->type == ENEMY_TURRET_UP || enemy->type == ENEMY_MOVING_ROCK_UP)
                {
                    if (enemy->object.position.y >= player.object.position.y)
                    {
                        if (difference(enemy->object.position.x, player.object.position.x) < 12)
                        {
                            // Only spawn a projectile if the enemy doesn't currently have an active projectile.
                            if (enemy->projectileIndex == 0)
                            {
                                // Spawn projectile at the enemy turret position that is travelling downward.
                                spawnEnemyProjectile(enemy, i);
                            }
                            else if (enemy->type == ENEMY_MOVING_ROCK_UP)
                            {
                                enemy->object.velocity.y = -6;
                            }
                        }
                    }
                }
                else if (enemy->type == ENEMY_TIMED_SPIKE)
                {
                    enemy->logicFlag++;

                    // Timed spike goes up and down at an arbitrary interval.
                    // TODO tune this as required.
                    if (enemy->logicFlag > 128)
                    {
                        // Activate the timed spike interactable at the enemy timed spike position (makes it go up and down).
                        activateInteractable(enemy->object.position.x / TILE_WIDTH_BYTES, enemy->object.position.y / TILE_HEIGHT, currentRoomX, currentRoomY, 0);

                        // Reset flag for next interval.
                        enemy->logicFlag = 0;
                    }
                }
            }

            // Enemies only move once per logic flag so only adjusting direction once per logic flag cycle.
            if (!enemy->logicFlag && enemy->type == ENEMY_SHARK)
            {
                // Check if the player is water and runs logic accordingly.
                if (player.inWater)
                {
                    // Reset player velocity before determining new direction for enemy.
                    enemy->object.velocity.x = 0;
                    enemy->object.velocity.y = 0;

                    // Sharks follow the player around when in the water by check if the player is near
                    // on the x axis or the y axis.
                    if (difference(player.object.position.x, enemy->object.position.x) > 1)
                    {
                        if (player.object.position.x > enemy->object.position.x)
                        {
                            // Player to the right of enemy.
                            enemy->object.velocity.x = 2;
                        }
                        else if (player.object.position.x < enemy->object.position.x)
                        {
                            //Player to the left of enemy.
                            enemy->object.velocity.x = -2;
                        }
                    }
                    else if (difference(player.object.position.y, enemy->object.position.y) > 1)
                    {
                        if (player.object.position.y > enemy->object.position.y)
                        {
                            // Player below enemy.
                            enemy->object.velocity.y = 3;
                        }
                        else if (player.object.position.y < enemy->object.position.y)
                        {
                            // Player above enemy.
                            enemy->object.velocity.y = -3;
                        }
                    }
                }
                else
                {
                    // Player not in water so move around randomly.
                    // Set a pseudo random direction for the sharks when player not detected.
                    u8 randomDirection = cpct_rand() % 4;

                    if (randomDirection == DIRECTION_LEFT)
                    {
                        // Change direction to left.
                        enemy->object.velocity.y = 0;
                        enemy->object.velocity.x = -2;
                    }
                    else if (randomDirection == DIRECTION_UP)
                    {
                        // Change direction to up.
                        enemy->object.velocity.y = -2;
                        enemy->object.velocity.x = 0;
                    }
                    else if (randomDirection == DIRECTION_RIGHT)
                    {
                        // Change direction to right.
                        enemy->object.velocity.y = 0;
                        enemy->object.velocity.x = 2;
                    }
                    else if (randomDirection == DIRECTION_DOWN)
                    {
                        // Change direction to down.
                        enemy->object.velocity.y = 2;
                        enemy->object.velocity.x = 0;
                    }
                }
            }
        }

        ++enemy;
    }
}

void updateEnemyPosition(Enemy* enemy)
{
    // The tiles that the enemy currently occupies.
    u8 occupiedXTileLeft = (enemy->object.position.x) >> 2;
    u8 occupiedXTileRight = (enemy->object.position.x + TILE_WIDTH_BYTES - 1);
    u8 occupiedYTileBottom = (enemy->object.position.y + TILE_HEIGHT - 1);
    u8 occupiedYTileTop = (enemy->object.position.y) >> 4;
    occupiedXTileRight >>= 2;
    occupiedYTileBottom >>= 4;

    // Save the previous position of the enemy.
    enemy->object.previousPosition.x = enemy->object.position.x;
    enemy->object.previousPosition.y = enemy->object.position.y;

    // Enemy is currently assumed to only have horizontal or vertical movement, never both at the same time.
    if (enemy->object.velocity.x)
    {
        u8 futureXTileLeft = (enemy->object.position.x + enemy->object.velocity.x) >> 2;
        u8 futureXTileRight = (enemy->object.position.x + TILE_WIDTH_BYTES - 1 + enemy->object.velocity.x);
        futureXTileRight >>= 2;

        if (enemy->object.velocity.x > 0 && enemy->object.position.x + enemy->object.velocity.x <= SCREEN_X - TILE_WIDTH_BYTES)
        {
            // Enemy going right.
            if (processCellForEnemy(futureXTileRight, occupiedYTileTop, enemy) == CELL_EMPTY &&
                processCellForEnemy(futureXTileRight, occupiedYTileBottom, enemy) == CELL_EMPTY)
            {
                enemy->object.position.x += enemy->object.velocity.x;
            }
        }
        else if (enemy->object.velocity.x < 0 && enemy->object.position.x + enemy->object.velocity.x >= 0)
        {
            // Enemy going left.
            if (processCellForEnemy(futureXTileLeft, occupiedYTileTop, enemy) == CELL_EMPTY &&
                processCellForEnemy(futureXTileLeft, occupiedYTileBottom, enemy) == CELL_EMPTY)
            {
                enemy->object.position.x += enemy->object.velocity.x;
            }
        }
    }
    else if (enemy->object.velocity.y)
    {
        u8 futureYTileBottom = (enemy->object.position.y + TILE_HEIGHT - 1 + enemy->object.velocity.y);
        u8 futureYTileTop = (enemy->object.position.y + enemy->object.velocity.y) >> 4;
        futureYTileBottom >>= 4;

        if (enemy->object.velocity.y > 0 && enemy->object.position.y + enemy->object.velocity.y <= SCREEN_Y_NO_HUD - TILE_HEIGHT)
        {
            // Enemy going down.
            if (processCellForEnemy(occupiedXTileLeft, futureYTileBottom, enemy) == CELL_EMPTY &&
                processCellForEnemy(occupiedXTileRight, futureYTileBottom, enemy) == CELL_EMPTY)
            {
                enemy->object.position.y += enemy->object.velocity.y;
            }
        }
        else if (enemy->object.velocity.y < 0 && enemy->object.position.y + enemy->object.velocity.y >= 0)
        {
            // Enemy going up.
            if (processCellForEnemy(occupiedXTileLeft, futureYTileTop, enemy) == CELL_EMPTY &&
                processCellForEnemy(occupiedXTileRight, futureYTileTop, enemy) == CELL_EMPTY)
            {
                enemy->object.position.y += enemy->object.velocity.y;
            }
        }
    }

    // If the enemy has moved it needs to be redrawn.
    if (enemy->object.previousPosition.x != enemy->object.position.x ||
        enemy->object.previousPosition.y != enemy->object.position.y)
    {
        enemy->redraw = TRUE;
    }

    // Reset enemy velocity after potentially updating position.
    // For now this is commented out to get it working.
    // TODO improve this or make use of it somehow.
    //enemy->object.velocity.x = 0;
    //enemy->object.velocity.y = 0;
}

u8 processCellForEnemy(u8 x, u8 y, Enemy* enemy)
{
    u8 cell = mapCell(x, y, currentRoomX, currentRoomY);
    u8 cellType = cell & CELL_GFX_RANGE;

    // Check what type of cell it is.
    if (cell & CELL_ENVIRONMENT_FLAG)
    {
        // Sharks can move in water but not other environment cells.
        if (enemy->type == ENEMY_SHARK)
        {
            if (cellType == ENVIRONMENT_WATER)
            {
                return 0;
            }
            else
            {
                return 1;
            }
        }

        // Prevent the enemy from going onto a chasm cell or any potentially solid environment cells.
        // Dangerous plants are treated as solid to block enemies.
        if (cellType >= ENVIRONMENT_DANGEROUS_PLANT)
        {
            return 1;
        }

        // Enemies can move along environment cells.
        return 0;
    }
    else if (cell & CELL_INTERACTABLE_FLAG && cellType == INTERACTABLE_PRESSURE_PLATE_2)
    {
        // This is facilitate an enemy activating a pressure plate.
        if (positionContainsLink(x, y))
        {
            // I tried adding the switch sound effect here but it was a bit obnoxious
            // having it activate many times as the enemy walks up and down.

            // Save the source position of the link which the enemy is currently on.
            enemy->activatedLinkX = x;
            enemy->activatedLinkY = y;
        }

        return 0;
    }
    else if (cell & CELL_INTERACTABLE_FLAG && cellType == INTERACTABLE_PHASE_BLOCK_ON)
    {
        // Enemies can move onto active phase blocks such as if the player has just teleported in.
        return 0;
    }

    // Everything whether solid or not will be returned as (items and interactables treated as solid).
    return cell;
}

void convertEnemyToSpike(Enemy* enemy)
{
    u8 x = enemy->object.position.x / TILE_WIDTH_BYTES;
    u8 y = enemy->object.position.y / TILE_HEIGHT;

    u8 cell = mapCell(x, y, currentRoomX, currentRoomY);

    if (cell != CELL_INTERACTABLE_FLAG + INTERACTABLE_PHASE_BLOCK_ON)
    {
        // Convert cell at enemy position to timed spike.
        updateMapCellRedraw(x, y, currentRoomX, currentRoomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_TIMED_SPIKE_DOWN);

        // Velocity must be 0 for timed spike to work.
        enemy->object.velocity.x = 0;
        enemy->object.velocity.y = 0;

        // Change enemy type to timed spike.
        // This seems like an interesting thing to do with the moving rock after it has finished moving.
        enemy->type = ENEMY_TIMED_SPIKE;
    }
    else
    {
        // Destroy the enemy moving rock otherwise it will remove the phase block from the map which could break the level.
        destroyEnemy(enemy);
    }
}

void destroyEnemy(Enemy* enemy)
{
    u8 cellX = enemy->object.position.x / TILE_WIDTH_BYTES;
    u8 cellY = enemy->object.position.y / TILE_HEIGHT;
    backbufferWidth = cellX == 19 ? TILE_WIDTH_BYTES : BACKBUFFER_WIDTH;

    // Draw over where the enemy was using the backbuffer.
    populateBackbufferFromMap(cellX, cellY);
    drawBackbuffer(cellX, cellY);

    // No longer draw the enemy.
    enemy->redraw = FALSE;

    // Move enemy off screen where it can never be drawn.
    // Then the player does not need to do a health check for enemies.
    enemy->object.position.room = 15;
}

u8 spawnEnemyProjectile(Enemy* enemy, u8 enemyIndex)
{
    // Get the next index in the projectile queue.
    u8 nextIndex = nextProjectileIndex();

    if (nextIndex)
    {
        // Create the new projectile at the specified position (of the enemy).
        initProjectile(&projectileState[nextIndex], enemy->object.position.x, enemy->object.position.y, enemy->direction);

        // Save the index of the projectile to the enemy.
        mapEnemies[enemyIndex].projectileIndex = nextIndex;

        // Save the id of the enemy to the projectile.
        projectileState[nextIndex].actorId = enemy->id;

        // Save the index in the projectile queue to the projectile.
        projectileState[nextIndex].projectileQueueIndex = nextIndex;

        // Mark the entry in the queue as spawned and save the index in the enemy array.
        projectileQueue[nextIndex] = enemyIndex;
    }

    return nextIndex;
}