#ifndef GAMESTATE_H
#define GAMESTATE_H

#include "Difficulty.h"
#include "GameTypes.h"

// Indicates if the game is currently running within the game loop.
extern u8 gameRunning;

// Indicates if the hud is redrawn on the screen.
extern u8 hudRedraw;

// Indicates that the time remaining has changed and is to be redrawn on the screen.
extern u8 timeRedraw;

// The score for the whole game session.
extern u16 gameScore;

// The score including the progress for the current level.
extern u16 levelScore;

// The current room that the player is in (0 to 8).
extern u8 currentRoom;

// The current room (x coordinate within the grid).
extern u8 currentRoomX;

// The current room (y coordinate within the grid).
extern u8 currentRoomY;

// Indicates the player has recently changed rooms.
extern u8 roomChange;

// Indicates the state of the level (or game) and whether it has ended (including success or failure).
extern u8 levelState;

// The number of lives that the player has.
extern u8 playerLives;

// The number of seconds remaining in the level.
extern u8 levelTimerSecondsRemaining;

// Indicates whether the level timer is enabled and displaying on screen.
extern u8 enableLevelTimer;

// The current identifier value used to uniquely identify actors within the game.
extern u8 currentId;

// Indicates whether the game was started using a pass code.
extern u8 passcodeUsed;

// The number of coins the player has collected in the game.
extern u8 coinCount;

// The X position (of the whole map) of the current checkpoint that the player has saved.
extern u8 checkpointX;

// The Y position (of the whole map) of the current checkpoint that the player has saved.
extern u8 checkpointY;

// Indicates whether the restore dialog should be shown when restoring last checkpoint.
extern u8 showRestoreDialog;

// Resets state values for the start of a new game session.
void resetGameSessionState();

#endif