#ifndef GAMETYPE_H
#define GAMETYPE_H

#include <types.h>

// Boolean true value 1.
#define TRUE 1

// Boolean false value 0.
#define FALSE 0

#ifndef WINCPCTELERA
#define NULL 0
#endif

// The number of cells in the map grid.
#define MAP_CELL_COUNT 2160

// The number of levels in the game.
#define LEVEL_COUNT 6

// Number of tiles that make up the screen vertically.
#define SCREEN_TILES_Y 12

// Number of tiles that make the screen horizontally.
#define SCREEN_TILES_X 20

// Number of tiles that make up the level vertically.
#define MAP_TILES_Y 36

// Number of tiles that make up the level horizontally.
#define MAP_TILES_X 60

// Screen y dimension only including tiles (hud height isn't included).
#define SCREEN_Y_NO_HUD 192

// Screen X dimension.
// Note: 80 = 160/2
#define SCREEN_X 80

// Screen y dimension.
#define SCREEN_Y 200

// X positions in (x,y) of cells in the grid.
extern const u8 xPositions[SCREEN_TILES_X];

// Y positions in (x,y) of cells in the grid.
extern const u8 yPositions[SCREEN_TILES_Y];

// Height of a tile on screen.
#define TILE_HEIGHT 16

// Width of a tile on screen.
#define TILE_WIDTH_BYTES 4

// Cell empty flag.
#define CELL_EMPTY 0

// Flag indicating the cell is an item.
#define CELL_ITEM_FLAG 0x20

// Flag indicating the cell is an interactable.
#define CELL_INTERACTABLE_FLAG 0x40

// Flag inticating the cell is an environment cell.
#define CELL_ENVIRONMENT_FLAG 0x80

// The possible bits that the cell graphic could be in.
#define CELL_GFX_RANGE 0x1F

// Ready to start the level.
#define LEVEL_READY 0

// The level has been completed successfully.
#define LEVEL_COMPLETE 1

// The level has been exited via the exit dialog.
#define LEVEL_EXIT 2

// The player has died in the level.
#define LEVEL_DEATH 3

// All the levels in the game have been completed.
#define LEVEL_GAME_COMPLETE 4

// Player has died in a level and ran out of lives.
#define LEVEL_GAME_OVER 5

// Right direction.
#define DIRECTION_RIGHT 0

// Left direction.
#define DIRECTION_LEFT 1

// Up direction.
#define DIRECTION_UP 2

// Down direction.
#define DIRECTION_DOWN 3

// The number of characters in a pass code.
#define PASS_CODE_LENGTH 5

#endif