#include "Hud.h"

// X position of the first part of the hud.
#define HUD_X_POSITION_ 0

// X position of the second part of the hud.
#define HUD_X_POSITION_TWO_ 64

// Y position of the hud.
#define HUD_Y_POSITION_ 192

// Width of a hud item.
#define HUD_ITEM_WIDTH_BYTES_ 3

// Width of a thin hud item.
#define HUD_ITEM_THIN_WIDTH_BYTES_ 2

// Width of the first part of the hud.
#define HUD_WIDTH_ONE_ 64

// Width of the second part of the hud.
#define HUD_WIDTH_TWO_ 16

// Height of the hud.
#define HUD_HEIGHT_ 8

// X position of the game score.
#define HUD_SCORE_START_ 0

// Hud health X position.
#define HEALTH_START_ 13

// Hud lives count X position.
#define LIVES_START_ 21

// Hud projectile weapon X position.
#define WEAPON_START_ 27

// "Hud artefact piece count X position.
#define ARTEFACT_PIECE_COUNT_ 35

// Hud red key X position.
#define RED_KEY_START_ 47

// Hud green key X position.
#define GREEN_KEY_START_ 49

// Hud yellow key X position.
#define YELLOW_KEY_START_ 51

// Hud machete X position.
#define MACHETE_START_ 53

// Hud fins X position.
#define FINS_START_ 56

// Hud block X position.
#define BLOCK_START_ 59

// Hud glasses X position.
#define GLASSES_START_ 62

// HUD_TIMER_TEXT_START_
#define HUD_TIMER_TEXT_START_ 74

// Indicates if the hud is redrawn on the screen.
u8 hudRedraw = TRUE;

// Indicates that the time remaining has changed and is to be redrawn on the screen.
u8 timeRedraw = TRUE;

// Draws the game hud at the bottom of the screen.
void drawHud()
{
    if (hudRedraw)
    {
        static u8 hudText[6];

        // Draw the hud strip on the background in two parts.
        u8* memory = cpctm_screenPtr(CPCT_VMEM_START, HUD_X_POSITION_, HUD_Y_POSITION_);
        cpct_drawSolidBox(memory, cpctm_px2byteM0(2, 2), HUD_WIDTH_ONE_, HUD_HEIGHT_);
        cpct_drawSolidBox(memory + HUD_X_POSITION_TWO_, cpctm_px2byteM0(2, 2), HUD_WIDTH_TWO_, HUD_HEIGHT_);

        // Draw the current game score at the bottom left.
        drawScore(levelScore, HUD_SCORE_START_, HUD_Y_POSITION_);

        // Player current health.
        cpct_drawSprite(G_hud_health, memory + HEALTH_START_, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        convertNumberToString(player.health, hudText);
        drawString(HEALTH_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);

        // Player lives count.
        cpct_drawSprite(G_hud_lives, memory + LIVES_START_, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        convertNumberToString(playerLives, hudText);
        drawString(LIVES_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);

        // Hud artefact (staff) piece count.
        cpct_drawSprite(G_hud_staff, memory + ARTEFACT_PIECE_COUNT_, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        convertNumberToString(remainingArtefactPieces, hudText);
        drawString(ARTEFACT_PIECE_COUNT_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);

        // Player inventory items.
        if (inventoryItems & INVENTORY_ITEM_KEY_RED)
        {
            memory = cpctm_screenPtr(CPCT_VMEM_START, RED_KEY_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_key_red, memory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        if (inventoryItems & INVENTORY_ITEM_KEY_GREEN)
        {
            memory = cpctm_screenPtr(CPCT_VMEM_START, GREEN_KEY_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_key_green, memory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        if (inventoryItems & INVENTORY_ITEM_KEY_YELLOW)
        {
            memory = cpctm_screenPtr(CPCT_VMEM_START, YELLOW_KEY_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_key_yellow, memory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        if (inventoryItems & INVENTORY_ITEM_BOW)
        {
            memory = cpctm_screenPtr(CPCT_VMEM_START, WEAPON_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_bow, memory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);

            // Projectile ammo count.
            convertNumberToString(player.ammo, hudText);
            drawString(WEAPON_START_ + 3, HUD_Y_POSITION_, hudText);
        }

        if (inventoryItems & INVENTORY_ITEM_MACHETE)
        {
            memory = cpctm_screenPtr(CPCT_VMEM_START, MACHETE_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_machete, memory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        if (inventoryItems & INVENTORY_ITEM_FINS)
        {
            memory = cpctm_screenPtr(CPCT_VMEM_START, FINS_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_fins, memory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        if (inventoryItems & INVENTORY_ITEM_BLOCK)
        {
            memory = cpctm_screenPtr(CPCT_VMEM_START, BLOCK_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_block, memory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        if (inventoryItems & INVENTORY_ITEM_GLASSES)
        {
            memory = cpctm_screenPtr(CPCT_VMEM_START, GLASSES_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_glasses, memory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        // The level timer needs to be redrawn as the entire hud has been redrawn.
        timeRedraw = TRUE;

        // Hud will not redraw until hudRedraw is set to true for the next update.
        hudRedraw = FALSE;
    }

    if (timeRedraw)
    {
        drawTimeRemaining(levelTimerSecondsRemaining, HUD_TIMER_TEXT_START_, HUD_Y_POSITION_);

        // We don't want to draw the time continuously.
        timeRedraw = FALSE;
    }
}

void updateBorder()
{
    if (!remainingArtefactPieces)
    {
        cpct_setBorder(HW_ORANGE);
    }
}