#include "Interactable.h"

u8 isInteractableReady(u8 interactable)
{
    switch (interactable)
    {
        // Make sure the red door has a matching red key.
        case INTERACTABLE_DOOR_RED:
        {
            if (inventoryItems & INVENTORY_ITEM_KEY_RED)
            {
                // Remove the red key from the inventory on use.
                inventoryItems &= ~INVENTORY_ITEM_KEY_RED;

                return TRUE;
            }

            return FALSE;
        }

        // Make sure the green door has a matching green key.
        case INTERACTABLE_DOOR_GREEN:
        {
            if (inventoryItems & INVENTORY_ITEM_KEY_GREEN)
            {
                // Remove the green key from the inventory on use.
                inventoryItems &= ~INVENTORY_ITEM_KEY_GREEN;

                return TRUE;
            }

            return FALSE;
        }

        // Make sure the yellow door has a matching yellow key.
        case INTERACTABLE_DOOR_YELLOW:
        {
            if (inventoryItems & INVENTORY_ITEM_KEY_YELLOW)
            {
                // Remove the green key from the inventory on use.
                inventoryItems &= ~INVENTORY_ITEM_KEY_YELLOW;

                return TRUE;
            }
        }

        // I am abusing fall through to cause it to return FALSE if the player is not carrying the appropriate key for the yellow door.

        // Gate is the only other interactable that will be false.
        case INTERACTABLE_GATE:
        {
            return FALSE;
        }
        break;
    }

    // Everything that is not a door or a gate is ready by default.
    return TRUE;
}

u8 activateInteractable(u8 x, u8 y, u8 roomX, u8 roomY, u8 sourceInteractbleType)
{
    u8 cell = mapCell(x, y, roomX, roomY);

    // Only activate a cell if it is an interactable.
    if (cell & CELL_INTERACTABLE_FLAG)
    {
        u8 cellType = cell & CELL_GFX_RANGE;

        if (cellType == INTERACTABLE_EXIT)
        {
            if (!remainingArtefactPieces)
            {
                levelState = LEVEL_COMPLETE;
                gameRunning = FALSE;
                return TRUE;
            }
            else
            {
                // Artefact dialog only shown if there are pieces remaining on the map.
                artefactPieceDialog();
                return FALSE;
            }
        }
        else if (cellType == INTERACTABLE_GATE)
        {
            // Change the gate to empty on activation.
            // I might be running out of interactables so using an environment cell for now.
            // Redraw the cell where the gate was as it is now clear.
            updateMapCellRedraw(x, y, roomX, roomY, CELL_ENVIRONMENT_FLAG + ENVIRONMENT_GATE_OPEN);
        }
        else if (cellType == INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_LEFT)
        {
            // Change the switchable one way block from left to right.
            // Redraw the cell where the switchable block has changed direction.
            updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_RIGHT);
        }
        else if (cellType == INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_RIGHT)
        {
            // Change the switchable one way block from right to left.
            // Redraw the cell where the switchable block has changed direction.
            updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_LEFT);
        }
        else if (cellType == INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_UP)
        {
            // Change the switchable one way block from up to down.
            // Redraw the cell where the switchable block has changed direction.
            updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_DOWN);
        }
        else if (cellType == INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_DOWN)
        {
            // Change the switchable one way block from down to up.
            // Redraw the cell where the switchable block has changed direction.
            updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_UP);
        }
        else if (cellType == INTERACTABLE_LEVER_OFF)
        {
            // Change the lever from off to on.
            updateMapCell(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_LEVER_ON);

            // Redraw the player so they don't disappear behind the lever.
            player.redraw = TRUE;

            // Play the switch sound effect on using the lever.
            playSoundEffect(SFX_SWTICH, CHANNEL_A, 4);

            // Activate the interactable linked to this one using the source coordinates of the link.
            activateLinkedInteractable(x, y);
        }
        else if (cellType == INTERACTABLE_LEVER_ON)
        {
            // Change the lever from on to off.
            updateMapCell(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_LEVER_OFF);

            // Redraw the player so they don't disappear behind the lever.
            player.redraw = TRUE;

            // Play the switch sound effect on using the lever.
            playSoundEffect(SFX_SWTICH, CHANNEL_A, 4);

            // Activate the interactable linked to this one using the source coordinates of the link.
            activateLinkedInteractable(x, y);
        }
        else if (cellType == INTERACTABLE_TOGGLE_SPIKE_DOWN)
        {
            // Toggle spike should only be activated from a linked interactable.
            if (sourceInteractbleType)
            {
                // Change the spike from down to up.
                // Redraw the cell where the up spike.
                updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_TOGGLE_SPIKE_UP);
            }
        }
        else if (cellType == INTERACTABLE_TOGGLE_SPIKE_UP)
        {
            // Toggle spike should only be activated from a linked interactable.
            if (sourceInteractbleType)
            {
                // Change the spike from up to down.
                // Redraw the cell where the up spike.
                updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_TOGGLE_SPIKE_DOWN);
            }
        }
        else if (cellType == INTERACTABLE_TIMED_SPIKE_DOWN)
        {
            // Change the spike from down to up.
            // Redraw the cell where the up spike.
            updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_TIMED_SPIKE_UP);

            // Check if the player is currently on top of the previously down spike.
            if (difference(y * TILE_HEIGHT, player.object.position.y) < TILE_HEIGHT &&
                difference(x * TILE_WIDTH_BYTES, player.object.position.x) < TILE_WIDTH_BYTES)
            {
                // The player is stationed on the spike so trigger a spike detection in the player.
                player.spikeDetected = TRUE;
                player.redraw = TRUE;
            }
        }
        else if (cellType == INTERACTABLE_TIMED_SPIKE_UP)
        {
            // Change the spike from up to down.
            // Redraw the cell where the up spike.
            updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_TIMED_SPIKE_DOWN);
        }
        else if (cellType == INTERACTABLE_LADDER || cellType == INTERACTABLE_PHASE_BLOCK_ON || cellType == INTERACTABLE_PHASE_BLOCK_OFF || cellType == INTERACTABLE_TRAPDOOR)
        {
            // Check that the source interactable is from a player activated interactble
            // where it will be 0.
            if (sourceInteractbleType == 0)
            {
                // The room matches the current room so this is the source ladder or phase block.
                activateLinkedInteractable(x, y);
            }
            else if (sourceInteractbleType == INTERACTABLE_LADDER || sourceInteractbleType == INTERACTABLE_PHASE_BLOCK_ON || sourceInteractbleType == INTERACTABLE_TRAPDOOR)
            {
                // When doing a teleport we need to make sure the source is either a ladder or an active phase block.
                // In inactive phase block cannot be used to teleport.
                teleportPlayer(x, y, roomX, roomY);
            }
            else if (sourceInteractbleType == INTERACTABLE_LEVER_ON ||
                     sourceInteractbleType == INTERACTABLE_PRESSURE_PLATE_2 ||
                     sourceInteractbleType == CELL_ENVIRONMENT_FLAG + ENVIRONMENT_HIDDEN_TRIGGER)
            {
                // Happy accident that environment trigger works so well as only the interactable flag is removed from sourceInteractbleType.
                // Update a phase block from off to on when activated by lever, pressure plate or hidden trigger.
                updateMapCellRedraw(x, y, roomX, roomY, INTERACTABLE_PHASE_BLOCK_ON | CELL_INTERACTABLE_FLAG);
            }
        }
        else if (cellType == INTERACTABLE_PRESSURE_PLATE_2_BLOCK)
        {
            // Play the switch sound effect on activating the pressure plate with a block.
            playSoundEffect(SFX_SWTICH, CHANNEL_A, 4);

            // Activates the interactable that is linked to the pressure plate.
            activateLinkedInteractable(x, y);
        }
        else if (cellType == INTERACTABLE_CHECKPOINT)
        {
            // Checkpoints are only active for easy and normal difficulty.
            if (checkpointsActive)
            {
                // Play the switch sound effect on using the checkpoint.
                playSoundEffect(SFX_SWTICH, CHANNEL_A, 4);

                // Change checkpoint to active on use.
                updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_CHECKPOINT_ACTIVE);

                // Check if a checkpoint has previously been activated.
                if (checkpointX != 255)
                {
                    // Reset the previously activated checkpoint.
                    // This will redraw the cell containing the checkpoint in case the previous checkpoint is in the same room.
                    updateMapCellRedraw(checkpointX % SCREEN_TILES_X, checkpointY % SCREEN_TILES_Y, checkpointX / SCREEN_TILES_X, checkpointY / SCREEN_TILES_Y, CELL_INTERACTABLE_FLAG + INTERACTABLE_CHECKPOINT);
                }

                // Update checkpoint location and room on activating the new checkpoint.
                checkpointX = x + (SCREEN_TILES_X * currentRoomX);
                checkpointY = y + (SCREEN_TILES_Y * currentRoomY);

                // Redraw the player so they don't disappear behind the checkpoint.
                player.redraw = TRUE;
            }
        }

        return TRUE;
    }
    else if (cell == CELL_ENVIRONMENT_FLAG + ENVIRONMENT_LAVA_FLOW)
    {
        // Make sure that a flow can only be activated from a linked interactable.
        if (sourceInteractbleType)
        {
            // Convert the linked lava flow cell and connected cells to lava.
            triggerFlow(x, y, roomX, roomY, ENVIRONMENT_LAVA_FLOW + CELL_ENVIRONMENT_FLAG, CELL_ENVIRONMENT_FLAG | ENVIRONMENT_LAVA);
        }
    }
    else if (cell == CELL_ENVIRONMENT_FLAG + ENVIRONMENT_WALL_FLOW)
    {
        // Make sure that a flow can only be activated from a linked interactable.
        if (sourceInteractbleType)
        {
            // Convert the linked wall flow cell and connected cells to wall.
            triggerFlow(x, y, roomX, roomY, ENVIRONMENT_WALL_FLOW + CELL_ENVIRONMENT_FLAG, 1);
        }
    }
    else if (cell == CELL_ENVIRONMENT_FLAG + ENVIRONMENT_WATER_FLOW)
    {
        // Make sure that a flow can only be activated from a linked interactable.
        if (sourceInteractbleType)
        {
            // Convert the linked water flow cell and connected cells to water.
            triggerFlow(x, y, roomX, roomY, ENVIRONMENT_WATER_FLOW + CELL_ENVIRONMENT_FLAG, CELL_ENVIRONMENT_FLAG + ENVIRONMENT_WATER);
        }
    }
    else if (cell == CELL_ENVIRONMENT_FLAG + ENVIRONMENT_GATE_OPEN)
    {
        // Can only shut the gate from a source interactable, player cannot activate cell with use key.
        if (sourceInteractbleType)
        {
            // Change the open gate back to a closed gate.
            // I might be running out of interactables so using an environment cell for now.
            updateMapCellRedraw(x, y, roomX, roomY, CELL_INTERACTABLE_FLAG + INTERACTABLE_GATE);
        }
    }
    else if (cell == CELL_ENVIRONMENT_FLAG + ENVIRONMENT_TRAPDOOR_TARGET)
    {
        // Teleports the player from the trapdoor to the target.
        teleportPlayer(x, y, roomX, roomY);
    }

    // No interactable found so return false.
    return FALSE;
}

void activateLinkedInteractable(u8 sourceX, u8 sourceY)
{
    u8 i = 0;

    // Get the source interactable type (without the interactable flag) from the map.
    u8 sourceInteractable = mapCell(sourceX, sourceY, currentRoomX, currentRoomY) & ~CELL_INTERACTABLE_FLAG;;

    sourceX += SCREEN_TILES_X * currentRoomX;
    sourceY += SCREEN_TILES_Y * currentRoomY;

    for (i = 0; i < level_links_array_size[currentMap]; i += 4)
    {
        u8 x = level_links[currentMap][i];
        u8 y = level_links[currentMap][i + 1];

        if (sourceX == x && sourceY == y)
        {
            u8 destinationX = level_links[currentMap][i + 2];
            u8 destinationY = level_links[currentMap][i + 3];
            u8 linkedRoomX = destinationX / SCREEN_TILES_X;
            u8 linkedRoomY = destinationY / SCREEN_TILES_Y;

            // The destination should be inside the room rather than the full size.
            destinationX %= SCREEN_TILES_X;
            destinationY %= SCREEN_TILES_Y;

            // Activate the interactable at the destination end of the link.
            activateInteractable(destinationX, destinationY, linkedRoomX, linkedRoomY, sourceInteractable);
        }
    }
}

u8 positionContainsLink(u8 x, u8 y)
{
    u8 i = 0;
    x += SCREEN_TILES_X * currentRoomX;
    y += SCREEN_TILES_Y * currentRoomY;

    for (i = 0; i < level_links_array_size[currentMap]; i += 4)
    {
        if (level_links[currentMap][i] == x && level_links[currentMap][i + 1] == y)
        {
            return TRUE;
        }
    }

    return FALSE;
}