#ifndef INTERACTABLE_H
#define INTERACTABLE_H

#include <types.h>
#include "Dialog.h"
#include "GameState.h"
#include "GameTypes.h"
#include "Inventory.h"
#include "Map.h"
#include "Player.h"

#define INTERACTABLE_DOOR_RED 1
#define INTERACTABLE_DOOR_GREEN 2
#define INTERACTABLE_DOOR_YELLOW 3
#define INTERACTABLE_EXIT 4
#define INTERACTABLE_SPIKE_UP_LAND 5
#define INTERACTABLE_GATE 6
#define INTERACTABLE_ONE_WAY_BLOCK_RIGHT 7
#define INTERACTABLE_ONE_WAY_BLOCK_LEFT 8
#define INTERACTABLE_ONE_WAY_BLOCK_UP 9
#define INTERACTABLE_ONE_WAY_BLOCK_DOWN 10
#define INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_RIGHT 11
#define INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_LEFT 12
#define INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_UP 13
#define INTERACTABLE_SWITCHABLE_ONE_WAY_BLOCK_DOWN 14
#define INTERACTABLE_LEVER_ON 15
#define INTERACTABLE_LEVER_OFF 16
#define INTERACTABLE_LADDER 17
#define INTERACTABLE_PHASE_BLOCK_ON 18
#define INTERACTABLE_PHASE_BLOCK_OFF 19
#define INTERACTABLE_SPIKE_TRAP 20
#define INTERACTABLE_SPIKE_TRAP_ACTIVATED 21
#define INTERACTABLE_PRESSURE_PLATE_2 22
#define INTERACTABLE_TOGGLE_SPIKE_DOWN 23
#define INTERACTABLE_TOGGLE_SPIKE_UP 24
#define INTERACTABLE_TIMED_SPIKE_DOWN 25
#define INTERACTABLE_TIMED_SPIKE_UP 26
#define INTERACTABLE_TRAPDOOR 27
#define INTERACTABLE_SEA_URCHIN 28
#define INTERACTABLE_PRESSURE_PLATE_2_BLOCK 29
#define INTERACTABLE_CHECKPOINT 30
#define INTERACTABLE_CHECKPOINT_ACTIVE 31

// No matching link found in the link array.
#define NO_LINK_FOUND 255

// Tests if an interactable is ready for it to be used or activated.
u8 isInteractableReady(u8 interactable);

// Activates an interactable in a particular cell and room (typically where the player is).
u8 activateInteractable(u8 x, u8 y, u8 roomX, u8 roomY, u8 sourceInteractbleType);

// Activates a linked interactable from a specified source interactable position.
void activateLinkedInteractable(u8 sourceX, u8 sourceY);

// Indicates whether the specified x,y position is the source of a link.
u8 positionContainsLink(u8 x, u8 y);

#endif