#include "Interrupt.h"

// The current count of the collision timer when the player is invincible due to an enemy collision.
u16 collisionTimerCount = 0;

// Indicates that the collision timer is active.
u8 collisionActive = FALSE;

// The current count of the crumbling block timer used for transitioning between crumbling environment blocks.
u16 crumblingTimerCount = 0;

// Indicates that the crumbling timer is active.
u8 crumblingTimerActive = FALSE;

// Used to ensure the sound and music plays at the right speed.
u8 soundCounter_ = 0;

// The colours to cycle through to animate the lava.
// It cycles from bright to dark and then bright again.
static const u8 lavaCycle_[4] = { HW_BRIGHT_YELLOW, HW_ORANGE, HW_RED, HW_ORANGE };

// The colours to cycle through to animate water.
static const u8 waterCycle_[4] = { HW_BRIGHT_WHITE, HW_PASTEL_BLUE, HW_BLUE, HW_WHITE };

// The position within the lava cycle array for updating the palette.
u8 lavaCyclePosition_ = 0;

// Timer used for animating the lava using a palette swap.
u16 lavaPaletteTimer_ = 0;

// Timer used for the level timer count down.
u16 levelTimerCount_ = 0;

// The current count of the hedge timer used for transitioning between hedge states.
u16 hedgeTimerCount = 0;

// Indicates where the hedge timer is active.
u8 hedgeTimerActive = FALSE;

void interruptHandler()
{
    // Scan for any keyboard changes.
    cpct_scanKeyboard_if();

    if (collisionActive)
    {
        if (collisionTimerCount++ > 320)
        {
            collisionActive = FALSE;
        }
    }

    // Every 600 iterations of the interrupt handler for updating the lava colour will run.
    if (lavaPaletteTimer_++ > 500)
    {
        // Iterate through positions in the lava cycle array.
        lavaCyclePosition_ = lavaCyclePosition_ > 2 ? 0 : lavaCyclePosition_ + 1;

        // Set pen 14 to the colour which changes the speckles within the lava.
        cpct_setPALColour(14, lavaCycle_[lavaCyclePosition_]);

        // Set pen 15 to the colour which changes the speckles within the water.
        cpct_setPALColour(15, waterCycle_[lavaCyclePosition_]);

        // Reset timer after updating colour.
        lavaPaletteTimer_ = 0;
    }

    // Count down the level timer only when the timer is enabled.
    // It can be disabled if by difficulty or when the game is paused by a dialog.
    if (enableLevelTimer && levelTimerCount_++ > 410)
    {
        levelTimerCount_ = 0;

        // Count down one second.
        if (levelTimerSecondsRemaining)
        {
            --levelTimerSecondsRemaining;

            // Updates the game timer on the hud.
            timeRedraw = TRUE;
        }
    }

    // Used as time between crumbling stages for crumbling blocks.
    if (crumblingTimerActive && crumblingTimerCount++ > 93)
    {
        crumblingTimerActive = FALSE;
        crumblingTimerCount = 0;

        transitionCrumblingBlock();
    }

    // Timer required for the machete to take effect on the hedge cell.
    if (hedgeTimerActive && hedgeTimerCount++ > 320)
    {
        // Updates the hedge cell as triggered by the machete.
        updateHedgeCell();

        // Stop the hedge timer following the update of the hedge cell.
        stopHedgeTimer();
    }

    // Sound is played every 5 interrupt to get the speed right.
    if (soundMode == SOUND_MODE_ON && soundCounter_++ > 4)
    {
        playSound();
        soundCounter_ = 0;
    }
}

void activateCollisionTimer()
{
    if (!collisionActive)
    {
        collisionActive = TRUE;
        collisionTimerCount = 0;
    }
}