#ifndef MAP_H
#define MAP_H

#include <cpctelera.h>
#include "Enemy.h"
#include "GameState.h"
#include "GameTypes.h"
#include "Inventory.h"
#include "MapManager.h"
#include "Palette.h"
#include "sprites/EnvironmentSprites.h"
#include "sprites/InteractableSprites.h"
#include "sprites/ItemSprites.h"
#include "sprites/TileSprites.h"

// The size of the back buffer.
#define BACKBUFFER_SIZE 256

// The default width of the back buffer (two cells wide).
#define BACKBUFFER_WIDTH 8

// The default height of the back buffer (two cells tall).
#define BACKBUFFER_HEIGHT 32

// The map grid.
extern u8 mapCells[MAP_CELL_COUNT];

// Determine when a map redraw should happen.
extern u8 mapRedraw;

// The map index of the current map in the maps array.
extern u8 currentMap;

// The back buffer where tiles and actors are combined before drawing.
extern u8 backbuffer[BACKBUFFER_SIZE];

// The actual width of the back buffer where tiles and actors are combined before drawing.
extern u8 backbufferWidth;

// The environment cell type (including flag) used by default in the map grid.
extern u8 defaultEnvironmentCellType;

// The cell type (including flag) used when clearing a cell in the map grid.
extern u8 defaultClearCellType;

// The default cell in the map grid (includes flag).
#define DEFAULT_CELL defaultEnvironmentCellType

// The clear cell in the map grid (includes flag).
#define CLEAR_CELL defaultClearCellType

// The number of artefact pieces that the player needs to collect to complete the level.
extern u8 remainingArtefactPieces;

// The cell that the crumbling blocks will crumble to at the end of the crumble cycle.
extern u8 endCrumbleCell;

// Initialises the map grid.
void initMap();

// Draws the contents of the map on screen.
void drawMap();

// Draws the content of a cell from the map on screen.
void drawCell(u8 x, u8 y, u8 cell);

// Draws the cell at the location on the map on screen as a blended sprite.
void drawBlendedCell(u8 x, u8 y, u8 cell);

// Retrieve the index in the mapCell array for the given position (x,y) and room (x,y).
u16 mapCellPosition(u8 x, u8 y, u8 roomX, u8 roomY);

// Retrieve the cell from the mapCell array for the given position (x,y) and room (x,y).
u8 mapCell(u8 x, u8 y, u8 roomX, u8 roomY);

// Add the specified sprite to the backbuffer, dimensions are assumed to be the default TILE_WIDTH_BYTES * TILE_HEIGHT.
void addSpriteToBackbuffer(u8 x, u8 y, u8* sprite);

// Draws the content of the backbuffer at the specified location on the screen.
void drawBackbuffer(u8 x, u8 y);

// Populates the backbuffer with 4 cells from the map at the specified position.
void populateBackbufferFromMap(u8 cellX, u8 cellY);

// Retrieves a pointer to a graphic from a given cell value.
u8* graphicFromCell(u8 cell);

// Updates the cell in the mapCell array at the given position (x,y) and room (x,y).
void updateMapCell(u8 x, u8 y, u8 roomX, u8 roomY, u8 cell);

// Draws the cell at the specified position in the map on screen.
void drawCellAtPosition(u8 x, u8 y);

// Updates the cell in the mapCell array at the given position (x,y) and room (x,y) and redraw the cell if it is in the current room.
void updateMapCellRedraw(u8 x, u8 y, u8 roomX, u8 roomY, u8 cell);

// Checks whether the cell contains the specified cell type (type must include flag).
u8 cellContainsType(u8 x, u8 y, u8 type);

#endif