#ifndef PROJECTILE_H
#define PROJECTILE_H

#include <cpctelera.h>
#include "Interactable.h"
#include "Map.h"
#include "Object.h"

// Used when the projectile queue element has not been used (is free).
#define PROJECTILE_QUEUE_FREE 255

// Small moving object fired from the player's weapon.
typedef struct Projectile Projectile;
typedef struct Actor _Actor;

struct Projectile
{
    Object object;
    u8 backgroundSprite[8];
    u8 redraw;
    u8 direction;
    u8 health;
    u8 horizontalMovementFlag;

    // The id of the actor that spawned the projectile.
    u8 actorId;

    // The index of the projectile queue that the projectile belongs to.
    u8 projectileQueueIndex;
};

// Initialises values when constructing a projectile.
void initProjectile(Projectile* projectile, u8 x, u8 y, u8 direction);

// Draws the projectile.
void drawProjectile(Projectile* projectile);

// Copy the background that the projectile is currently occupying into the background sprite for later redraw.
void copyBackground(Projectile* projectile);

// Run the logic for the projectile.
void logicProjectile(Projectile* projectile);

// Destroys the projectile and cleans up the screen by drawing the projectile background.
void destroyProjectile(Projectile* projectile);

// Destroys any intersecting projectiles.
void processProjectileIntersection(Projectile* projectile);

// Projectile damages\destroys intersecting actors.
void actorProjectileColliction(Projectile* projectile);

// Damage the actor when it collides with the projectile.
void damageActor(_Actor* actor, Projectile* projectile);

// Test collision (intersection) between an actor and a projectile (velocity ignored).
u8 testActorProjectileIntersection(_Actor* actor, Projectile* projectile);

// Initialise the projectile state for all projectiles (player and enemy).
void initProjectiles();

#endif