#ifndef SOUND_H
#define SOUND_H

#include <cpctelera.h>
#include "arkos2/ArkosPlayer2.h"
#include "GameTypes.h"

// Must be defined in header file for wincpctelera.
#ifdef WINCPCTELERA
    extern void* OCTO_START;
    extern void* SEEKN_START;
    extern void* DIED2_START;
    extern void* FIN2_START;
    extern void* SFX_SOUNDEFFECTS;
#endif

// Sound effect when an item is picked up.
#define SFX_PICKUP 1

// Sound effect when the player takes damage.
#define SFX_TAKE_DAMAGE 2

// Sound effect when the player using a switch or lever.
#define SFX_SWTICH 3

// Sound effect when the player uses a teleporter (stairs, pressure plate or trapdoor).
#define SFX_TELEPORT 4

// Sound effect when the enemy takes damage from the player.
#define SFX_ENEMY_TAKE_DAMAGE 5

// Sound effect when the player walks over a crumbling block.
#define SFX_CRUMBLE 6

// Sound effect when the player opens a door.
#define SFX_DOOR 7

// Sound effect when the player shoots the projectile weapon.
#define SFX_SHOOT 8

// Sound effect when the player picks up a special item.
#define SFX_PICKUP_SPECIAL 9

// Sound effect when the player uses a machete on a hedge cell.
#define SFX_MACHETE 10

// Channel A of the AY.
#define CHANNEL_A 1

// Channel B of the AY.
#define CHANNEL_B 2

// Channel C of the AY.
#define CHANNEL_C 3

// The maximum volume to play sound effects at.
#define MAX_VOL 0

// Sound disabled.
#define SOUND_MODE_SOUND_DISABLED 0

// Sound effects and music enabled and sound turned off.
#define SOUND_MODE_OFF 1

// Sound effects and music enabled and sound turned on.
#define SOUND_MODE_ON 2

// Music played when the player loses the game.
#define MUSIC_DIED2 2

// Music played when the player finishes the game.
#define MUSIC_FIN2 3

// The current mode for playing sound (on, sound effects, music and sound effects).
extern u8 soundMode;

// Initialises all the arkos2 sound effects and music.
void initSound();

// Initialises the specified arkos2 sound effects data.
void initSoundEffects(void* soundData);

// Initialises the current arkos2 music data for playing (must be done before calling playMusic()).
void initMusic(void* musicData);

// Queues the specified sound effect and properties to be played on the next playSound() call.
void playSoundEffect(u8 effectNumber, u8 channel, u8 volume);

// Updates the current song to be played to the specified song number (this will be played on the next playSound() call).
// This allows us to support multiple songs within the arkos2 music data.
void playMusic(u8 songNumber);

// Players the currently selected arkos2 sound effects and music.
// This should be run in the interrupt at a regular interval.
void playSound();

// Triggers the sound top stop playing during the interrupt.
void stopSound();

#endif